package it.polito.esame.dao;


import it.polito.esame.bean.Composizione;
import it.polito.esame.bean.Formula;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class MolecoleDAO {
	
	// restituisce la formula che possiede come valore del campo idFormula 
	// quello passato in ingresso
	public Formula getFormulaById(int idFormula) {
		final String sql = "select idformula, nome, sigla " +
				"from formula " +
				"where idformula = ?";
		
		Formula f = null ;
		
		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);
			
			st.setInt(1, idFormula) ;

			ResultSet rs = st.executeQuery();

			if (rs.next()) {

				f = new Formula(
						rs.getInt("idformula"),
						rs.getString("nome"),
						rs.getString("sigla")
						) ;
			}

			rs.close();
			st.close();
			conn.close();

		} catch (SQLException e) {
			e.printStackTrace();
			f = null ;
		}

		return f;

	}
	
	// restituisce la formula che possiede come valore del campo nome  
	// quello passato in ingresso
	public Formula getFormulaByNome(String nome) {
		final String sql = "select idformula, nome, sigla " +
				"from formula " +
				"where nome = ?";
		
		Formula f = null ;
		
		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);
			
			st.setString(1, nome) ;

			ResultSet rs = st.executeQuery();

			if (rs.next()) {

				f = new Formula(
						rs.getInt("idformula"),
						rs.getString("nome"),
						rs.getString("sigla")
						) ;
			}

			rs.close();
			st.close();
			conn.close();

		} catch (SQLException e) {
			e.printStackTrace();
			f = null ;
		}

		return f;
	}
	
	// resituisce una lista di oggetti composizione in cui  presente come idMolecola
	// il valore che possiede idFormula dell'oggetto formula passato in ingresso
	public List<Composizione> getComposizione(Formula f) {
		final String sql = "select idElemento, quantita " +
				"from composizione " +
				"where idMolecola = ?";
		
		List<Composizione> lista = new ArrayList<>() ;
		
		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);
			
			st.setInt(1, f.getIdFormula()) ;

			ResultSet rs = st.executeQuery();

			while (rs.next()) {

				Composizione c = new Composizione(
						rs.getInt("idElemento"),
						rs.getInt("quantita")
						) ;
				
				lista.add(c) ;
			}

			rs.close();
			st.close();
			conn.close();

		} catch (SQLException e) {
			e.printStackTrace();
			lista = null ;
		}

		return lista ;
	}
	
}
