package it.polito.esame.dao;

import it.polito.esame.bean.Bridge;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class BridgeDAO {

	// restituisce l'elenco completo delle citt presenti nella tabella citta
	// ordinando il risultato in base al nome
	public ArrayList<String> getAllCities() {
		final String sql = "SELECT DISTINCT city " + "FROM citta "
				+ "ORDER BY city ASC";

		ArrayList<String> cities = new ArrayList<String>();

		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);

			ResultSet rs = st.executeQuery();

			while (rs.next()) {

				String city = rs.getString("city");
				cities.add(city);
			}

			rs.close();
			st.close();
			conn.close();

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return cities;

	}

	// restituisce un array di interi che rappresentano gli idPonte dei vari
	// ponti radio
	// che collegano le due citt specificate come parametro d'ingresso
	public ArrayList<Integer> getBridge(int source, int destination) {
		ArrayList<Integer> result = new ArrayList<Integer>();

		final String sql = "SELECT idPonte " + "FROM copertura "
				+ "WHERE idCitta=? AND idPonte IN " + "(SELECT idPonte "
				+ "FROM copertura " + "WHERE idCitta=? )";
		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);

			st.setInt(1, source);
			st.setInt(2, destination);

			ResultSet rs = st.executeQuery();

			while (rs.next()) {
				result.add(rs.getInt("idPonte"));
			}

			rs.close();
			st.close();
			conn.close();

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return result;

	}

	// restituisce l'oggetto Bridge che possiede come idPonte il valore passato in ingresso 
	public Bridge getBridgeById(int idPonte) {
		Bridge result = new Bridge();

		final String sql = "SELECT * " + "FROM ponte " + "WHERE idPonte =?";
		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);

			st.setInt(1, idPonte);

			ResultSet rs = st.executeQuery();

			while (rs.next()) {
				result.setIdPonte(idPonte);
				result.setOutput(rs.getString("output"));
				result.setCall(rs.getString("call"));
				result.setPl(rs.getFloat("pl"));
				result.setComments(rs.getString("comments"));
			}

			rs.close();
			st.close();
			conn.close();

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return result;

	}

	// restuisce l'idCitta della citt che possiede come valore del campo city
	// la stringa passata in ingresso
	public int getIdCittaByCity(String city) {
		int idCitta = -1;

		final String sql = "SELECT idCitta " + "FROM citta " + "WHERE city = ?";
		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);

			st.setString(1, city);

			ResultSet rs = st.executeQuery();

			while (rs.next()) {
				idCitta = rs.getInt("idCitta");
			}

			rs.close();
			st.close();
			conn.close();

		} catch (SQLException e) {
			e.printStackTrace();
			return idCitta;
		}

		return idCitta;

	}

}
