package it.polito.esame ;

import it.polito.esame.gui.Controller;
import it.polito.porto.bean.PortoArticle;
import it.polito.porto.bean.PortoCreator;
import it.polito.porto.dao.PortoDAO;

import java.io.IOException;

import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;

public class Esame extends Application {

	@Override
	public void start(Stage primaryStage) throws IOException {
		
		FXMLLoader loader = new FXMLLoader( this.getClass().getResource("gui/peers.fxml")) ;
		loader.load() ;
		
		
		Model model = new Model() ;
		loadModel(model) ;
		
		System.out.println("Building graph...") ;
		GraphModel graphModel = new GraphModel(model) ;
		System.out.println("Done.") ;
		
		Controller controller = loader.getController() ;
		controller.setModel(model) ;
		controller.setGraphModel(graphModel) ;
		
		Parent root = loader.getRoot() ;
		Scene scene = new Scene(root) ;
		primaryStage.setScene(scene) ;
		primaryStage.show() ;
		
	}
	
	private void loadModel(Model m) {
		PortoDAO dao = new PortoDAO() ;
		
		System.out.println("Loading data...") ;
		m.setCreators(dao.getAllCreator()) ;
		m.setArticles(dao.getAllArticle()) ; 
		
		System.out.println("Linking articles...") ;
		for( PortoArticle art: m.getArticles() ) {
			art.setCreators(dao.getCreatorsOfArticle(art)) ;
		}
		
		System.out.println("Linking authors...") ;
		for( PortoCreator cre: m.getCreators() ) {
			cre.setArticoli(dao.getArticlesOfCreator(cre)) ;
		}
		
		System.out.println("Done.") ;
	}


	public static void main(String[] args) {
		launch(args);
	}
}
