package it.polito.esame;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import it.polito.porto.bean.PortoArticle;
import it.polito.porto.bean.PortoCreator;

import org.jgrapht.Graphs;
import org.jgrapht.graph.DefaultEdge;
import org.jgrapht.graph.SimpleGraph;

public class GraphModel {
	
	private SimpleGraph<PortoCreator, DefaultEdge> graph ;
	
	
	public SimpleGraph<PortoCreator, DefaultEdge> getGraph() {
		return graph;
	}

	/**
	 * Builds the graph model of co-authorship. Vertices are authors. An edge between two authors 
	 * indicates that the two authors have coauthored at least one paper.
	 * 
	 * @param m the Model loaded from Porto from which the graph is created
	 */
	public GraphModel(Model m) {
		
		graph = new SimpleGraph<PortoCreator, DefaultEdge>(DefaultEdge.class) ;
		
		/* create a graph with all AUTHORS as nodes */
		Graphs.addAllVertices(graph, m.getCreators()) ;
		
		/* two authors are connected if there is at least one paper with their joint
		 * authorship.
		 * Navigate all papers, and enumerate all pairs of authors for each paper.
		 */
		for( PortoArticle art : m.getArticles() ) {
			// extract authors of current paper
			List<PortoCreator> authors = art.getCreators() ;
			
			for( int i = 0 ; i < authors.size() ; i++ ) {
				for( int j = i+1; j < authors.size() ; j++ ) {
					// all pairs (i,j) of authors, without repetitions
					PortoCreator ci = authors.get(i) ;
					PortoCreator cj = authors.get(j) ;
					
					graph.addEdge(ci, cj) ;
				}
			}
		}
	} // GraphModel(Model m)
	
	public Set<PortoCreator> getCoAuthors(Collection<PortoCreator> list) {
		Set<PortoCreator> coAuthors = new HashSet<PortoCreator>() ;
		
		for(PortoCreator author : list) {
			List<PortoCreator> coauthors = Graphs.neighborListOf(graph, author) ;
			coAuthors.addAll(coauthors) ;
		}
		
		return coAuthors ;
	}

}
